#!/bin/sh
kernelversion=2.6.21.4-kateos_smp

moduledep() {
    if [ ! -f "$2/lib/modules/$kernelversion/modules.dep" ]; then
	exit 1
    fi
    deps=$(awk 'BEGIN { searched=ARGV[2]; ARGV[2]=""; rc=1 } \
    function modname(filename) { match(filename, /\/([^\/]+)\.k?o(.gz):?$/, ret); return ret[1] } \
                function show() { if (orig == searched) { print dep; orig=""; rc=0; exit } } \
                /^\/lib/ { show(); \
                           orig=modname($1); dep=""; \
                           if ($2) { for (i = 2; i <= NF; i++) { dep=sprintf("%s %s", dep, modname($i)); } } } \
                /^	/ { dep=sprintf("%s %s", dep, modname($1));  } \
                END      { show(); exit(rc) }' $2/lib/modules/$kernelversion/modules.dep $1)
echo "$deps"
}

afterinstall(){

if [ -f /sbin/depmod ]; then
/sbin/depmod -a $kernelversion
fi


if test -f boot/initrd-smp ; then

if test ! -d tmp/mount ; then
mkdir tmp/mount
fi

if file boot/initrd-smp | grep gzip 1>/dev/null 2>/dev/null ; then 
GZIP=1
zcat boot/initrd-smp > tmp/initrd-smp
else
GZIP=0
cat boot/initrd-smp > tmp/initrd-smp
fi

CPIO=0
rcfile=linuxrc

if file tmp/initrd-smp | grep cpio 1>/dev/null 2>/dev/null ; then 
CPIO=1
rcfile=init
fi

if [ "$CPIO" != 1 ] ; then
mount -o loop tmp/initrd-smp tmp/mount
else
( cd tmp/mount ; cat ../initrd-smp | cpio -i -d -H newc --no-absolute-filenames 2>/dev/null )
fi

( cd tmp/mount/lib
for module in *.ko ; do
shortname=$(basename $module .ko)
rm $module
(cd ../../.. ; zcat `find lib/modules/$kernelversion/ -name $module.gz -type f` > tmp/mount/lib/$module) 

deps=$(moduledep $shortname ../../..)
if [ "$?" = "0" ] ; then
# We must be sure, that we don't need any additional dependencies!
for dep in $deps ; do
if [ ! -f $dep.ko ] ; then
(cd ../../.. ; zcat `find lib/modules/$kernelversion/ -name $dep.ko.gz -type f` > tmp/mount/lib/$dep.ko) 

# Yup, rcfile must be also changed ;)
(cd ../../.. ; cat tmp/mount/$rcfile | sed -e "s/insmod \/lib\/$module/insmod \/lib\/$dep.ko\ninsmod \/lib\/$module/g" > tmp/mount/$rcfile.new ; mv tmp/mount/$rcfile.new tmp/mount/$rcfile ; chmod +x tmp/mount/$rcfile) 
fi
done
fi

done
)

if [ "$CPIO" != 1 ] ; then
umount tmp/mount
else
rm -f tmp/initrd-smp
( cd tmp/mount ; find . | cpio -o -H newc > ../initrd-smp 2>/dev/null )
fi
rm -R tmp/mount

mv boot/initrd-smp boot/initrd-smp.old

if [ "$GZIP" = "1" ] ; then
gzip -9 tmp/initrd-smp
mv tmp/initrd-smp.gz boot/initrd-smp
else
mv tmp/initrd-smp boot/initrd-smp
fi
------------------------------------
else

if test -f boot/initrd ; then
# We must recreate initrd for smp kernel :P
if test ! -d tmp/mount ; then
mkdir tmp/mount
fi

if file boot/initrd | grep gzip 1>/dev/null 2>/dev/null ; then 
GZIP=1
zcat boot/initrd > tmp/initrd-smp
else
GZIP=0
cat boot/initrd > tmp/initrd-smp
fi

CPIO=0
rcfile=linuxrc

if file tmp/initrd-smp | grep cpio 1>/dev/null 2>/dev/null ; then 
CPIO=1
rcfile=init
fi

if [ "$CPIO" != 1 ] ; then
mount -o loop tmp/initrd-smp tmp/mount
else
( cd tmp/mount ; cat ../initrd-smp | cpio -i -d -H newc --no-absolute-filenames 2>/dev/null )
fi

( cd tmp/mount/lib
for module in *.ko ; do
shortname=$(basename $module .ko)
rm $module
(cd ../../.. ; zcat `find lib/modules/$kernelversion/ -name $module.gz -type f` > tmp/mount/lib/$module) 

deps=$(moduledep $shortname ../../..)
if [ "$?" = "0" ] ; then
# We must be sure, that we don't need any additional dependencies!
for dep in $deps ; do
if [ ! -f $dep.ko ] ; then
(cd ../../.. ; zcat `find lib/modules/$kernelversion/ -name $dep.ko.gz -type f` > tmp/mount/lib/$dep.ko) 

# Yup, rcfile must be also changed ;)
(cd ../../.. ; cat tmp/mount/$rcfile | sed -e "s/insmod \/lib\/$module/insmod \/lib\/$dep.ko\ninsmod \/lib\/$module/g" > tmp/mount/$rcfile.new ; mv tmp/mount/$rcfile.new tmp/mount/$rcfile ; chmod +x tmp/mount/$rcfile) 
fi
done
fi

done
)

if [ "$CPIO" != 1 ] ; then
umount tmp/mount
else
rm -f tmp/initrd-smp
( cd tmp/mount ; find . | cpio -o -H newc > ../initrd-smp 2>/dev/null )
fi
rm -R tmp/mount

if [ "$GZIP" = "1" ] ; then
gzip -9 tmp/initrd-smp
mv tmp/initrd-smp.gz boot/initrd-smp
else
mv tmp/initrd-smp boot/initrd-smp
fi

fi
fi

}

beforeremove(){
true
}

case "$1" in
'--afterinstall')
afterinstall
;;
'--beforeremove')
beforeremove
;;
*)
esac
